# 1) Prepare the file list
perl scripts/GenerateFileList.pl -d ./data -e wav -o preps/lists/wavelist.scp
perl scripts/GenerateFileList.pl -d ./data -e mfc -o preps/lists/mfclist.scp
perl scripts/Merge.pl -w preps\lists\wavelist.scp -m preps/lists/mfclist.scp -o preps/lists/list.scp

# 2) Prepare the Word level MLF from the file names
perl scripts/Scp2Mlf.pl -i preps/lists/wavelist.scp -o preps/word0.mlf

# 3) Prepare the lexicon
HDMan -m -w preps/wlist.txt -n preps/monophones.txt -l log/HDManlog.txt preps/lexicon.txt preps/dict.txt

# TO DO: Add the sentence boundary markers "SENT-START sil" and "SENT-END sil" to the lexicon

# 4) Prepare the phone level from word level mlfs
HLEd -T 1 -l '*' -d preps/lexicon.txt -i preps/phone0.mlf config/mkphones0.led preps/word0.mlf

# 5) Prepare the word lattice
HParse preps/grammar.bnf Preps/grammar.lat

# 6) Extract the feature vectors
HCopy -T 1 -F WAV -C config/HCopy.cfg -S preps/lists/list.scp

# 7) Intialize the flat start HMMs
HCompV -T 1 -C config/HCompV.cfg -f 0.01 -m -S preps/lists/mfclist.scp -M hmms/hmm0 preps/proto

# 8) Create HMMs for entries in the monophone list and replicate the global configuration
perl scripts/createMMF.pl -i hmms/hmm0/proto -m preps/monophones0.txt -o hmms/hmm0/hmmdefs.mmf
perl scripts/createMacros.pl -v hmms/hmm0/vFloors -o hmms/hmm0/macros

# 9) Training without additional transitions for sil
HERest -T 1 -C config/HERest.cfg -I preps/phone0.mlf -t 250.0 150.0 1000.0 -S preps/lists/mfclist.scp -H hmms/hmm0/macros -H hmms/hmm0/hmmdefs.mmf -M hmms/hmm1 preps/monophones0.txt
HERest -T 1 -C config/HERest.cfg -I preps/phone0.mlf -t 250.0 150.0 1000.0 -S preps/lists/mfclist.scp -H hmms/hmm1/macros -H hmms/hmm1/hmmdefs.mmf -M hmms/hmm2 preps/monophones0.txt
HERest -T 1 -C config/HERest.cfg -I preps/phone0.mlf -t 250.0 150.0 1000.0 -S preps/lists/mfclist.scp -H hmms/hmm2/macros -H hmms/hmm2/hmmdefs.mmf -M hmms/hmm3 preps/monophones0.txt

# 10) Add transitions to the sil model
HHEd -T 1 -H hmms/hmm3/macros -H hmms/hmm3/hmmdefs.mmf -M hmms/hmm4 config/sil.hed preps/monophones0.txt

# 11) Training with modified sil model
HERest -T 1 -C config/HERest.cfg -I preps/phone0.mlf -t 250.0 150.0 1000.0 -S preps/lists/mfclist.scp -H hmms/hmm4/macros -H hmms/hmm4/hmmdefs.mmf -M hmms/hmm5 preps/monophones0.txt
HERest -T 1 -C config/HERest.cfg -I preps/phone0.mlf -t 250.0 150.0 1000.0 -S preps/lists/mfclist.scp -H hmms/hmm5/macros -H hmms/hmm5/hmmdefs.mmf -M hmms/hmm6 preps/monophones0.txt

# 12) TO DO: Add an entry "SILENCE sil" to lexicon.txt

# 13) Recognize the training data to choose the appropriate transcription
HVite -T 1 -l '*' -o SWT -b SILENCE -C config/HVite.cfg -a -H hmms/hmm6/macros -H hmms/hmm6/hmmdefs.mmf -i preps/aligned.mlf -m -t 250.0 -y lab -I preps/word0.mlf -S preps/lists/mfclist.scp preps/lexicon.txt preps/monophones0.txt > log/HVitelog.txt

# 14) Prune the mfc list to exclude worst matching file
perl scripts/pruneMfc.pl -l log/HVitelog.txt -s preps/lists/mfclist_pr.scp

# 15) Train the models with the new aligned transcriptions
HERest -T 1 -C config/HERest.cfg -I preps/aligned.mlf -t 250.0 150.0 1000.0 -S preps/lists/mfclist_pr.scp -H hmms/hmm6/macros -H hmms/hmm6/hmmdefs.mmf -M hmms/hmm7 preps/monophones0.txt
HERest -T 1 -C config/HERest.cfg -I preps/aligned.mlf -t 250.0 150.0 1000.0 -S preps/lists/mfclist_pr.scp -H hmms/hmm7/macros -H hmms/hmm7/hmmdefs.mmf -M hmms/hmm8 preps/monophones0.txt

# 16) Test the data using Monophone models
HVite -T 1 -H hmms/hmm8/macros -H hmms/hmm8/hmmdefs.mmf -S preps/lists/mfclist_pr.scp -m -l '*' -i results/recMono.mlf -w preps/grammar.lat -p 0.0 -s 5.0 preps/lexicon.txt preps/monophones0.txt

# 17) Split the recognized output file into word and model level outputs
perl scripts/rec2WrdModel.pl -i results/recMono.mlf -w results/recMonoWord.mlf -m results/recMonoModel.mlf

# 18) Prepare the scores and confusion matrix
HResults -f -I preps/word0.mlf preps/monophones0.txt results/recMonoWord.mlf > results/Scores.Mono.txt
HResults -p -I preps/phone0.mlf preps/monophones0.txt results/recMonoModel.mlf > results/Confusion.Mono.txt

======================================================================================

Notes:
=====

1) sp phone is not used since the task is an isolatd word recognition
2) Training data is used for tesing for demonstration
3) Only monophones are used for recognition

Changes for Windows users:
=========================

1) Change '*' to * in steps 4, 13 and 16
2) Comment Linux labels and uncomment Windows portions in Scp2Mlf.pl, GenerateFileList.pl and GenerateWordList.pl
