# Author : Udhyakumar N
# Date   : 28-01-2004
# Notes  : To extract the Word and Model level output from recognized file

use strict;
use Getopt::Std;
getopts("i:w:m:");

if ( !defined ($Getopt::Std::opt_i) || !defined ($Getopt::Std::opt_w) || !defined ($Getopt::Std::opt_m) )
{
	print ( "Not all options defined \n" );
	print ( "Usage $0 -i InputRecfile -w WordRecfile -m ModelRecfile\n" );
	print ( "Eg. perl rec2WrdModel.pl -i recMono.txt -w recMonoWord.txt -m recMonoModel.txt" );
	die ( "\n" );
}

my ( $iRecfilename )      = $Getopt::Std::opt_i;
my ( $oWrdRecfilename )   = $Getopt::Std::opt_w;
my ( $oModelRecfilename ) = $Getopt::Std::opt_m;

unless ( open ( fInputRec, "$iRecfilename" ) )
{
	die  ("FATAL Error: Cannot open file $iRecfilename\n" );
}

unless ( open ( fRecWord, ">$oWrdRecfilename" ) ) 
{
	die  ("FATAL Error: Cannot open file $oWrdRecfilename \n" );
}

unless ( open ( fRecModel, ">$oModelRecfilename" ) )
{
	die  ("FATAL Error: Cannot open file $oModelRecfilename\n" );
}

my ( $recline, @recoutput );
my ( $samplestart, $sampleend, $recword );

$recline = <fInputRec>; #!MLF!#

print fRecWord  "#!MLF!#\n";
print fRecModel "#!MLF!#\n";

while ( $recline = <fInputRec> )
{
	# File name #
	print fRecModel $recline;
	print fRecWord  $recline;
	print STDOUT $recline;
	
	$recline = <fInputRec>;
	chomp ( $recline );
	@recoutput = split ( / +/, $recline );

	while ( $recline ne "." )
	{	
		$samplestart = $recoutput[0];
		$recword     = $recoutput[4];
		
		do
		{
			print fRecModel "$recoutput[0] $recoutput[1] $recoutput[2]\n";
			$sampleend = $recoutput[1];
			
			$recline = <fInputRec>;
			chomp ( $recline );	
			
			@recoutput = split ( / +/, $recline );
					
		} while ( $recoutput[4] eq "" && $recline ne "." );

		if ( $recword !~ /SENT/ && $recword !~ /ENTER/ && $recword !~ /EXIT/)
		{
			print fRecWord "$samplestart $sampleend $recword\n";
		}

	}

	print fRecWord  ".\n";
	print fRecModel ".\n";
}

close ( fInputRec );
close ( fRecWord  );
close ( fRecModel );
